/* 
   Copyright (C) 2006 Hans Fugal & Paul Davis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: /local/undo/libs/pbd3/pbd/undo.h 132 2006-06-29T18:45:16.609763Z fugalh  $
*/

#ifndef __lib_pbd_memento_command_h__
#define __lib_pbd_memento_command_h__

#include <iostream>
using std::cerr;
using std::endl;

#include <pbd/command.h>
#include <pbd/xml++.h>
#include <sigc++/slot.h>
#include <typeinfo>

/* grrr, strict C++ says that static member functions are not C functions, but we also want
   to be able to pack this into a sigc::ptr_fun and not sigc::mem_fun, so we have to make
   it a genuine function rather than a member.
*/

static void object_death (Command* mc) {
	delete mc;
}

/** This command class is initialized with before and after mementos 
 * (from Stateful::get_state()), so undo becomes restoring the before
 * memento, and redo is restoring the after memento.
 */

class PlaylistCommand : public Command
{
    public:
        PlaylistCommand(obj_T &object, 
                        UndoAction before,
			UndoAction after,
			
                       ) 
            : obj(object), before(before), after(after) {
		obj.GoingAway.connect (sigc::bind (sigc::ptr_fun (object_death), static_cast<Command*>(this)));
	}

	~PlaylistCommand () {
		GoingAway();
		if (before) {
			delete before;
		}
		if (after) {
			delete after;
		}
	}
        void operator() () 
        {
            if (after)
                obj.set_state(*after); 
        }
        void undo() 
        { 
            if (before)
                obj.set_state(*before); 
        }
        virtual XMLNode &get_state() 
        {
            string name;
            if (before && after)
                name = "PlaylistCommand";
            else if (before)
                name = "PlaylistUndoCommand";
            else
                name = "PlaylistRedoCommand";

	    
            XMLNode *node = new XMLNode(name);

	    node->add_property("obj_id", obj.id().to_s());
	    node->add_property("type_name", typeid(obj).name());
	    
	    if (before)
		    node->add_child_copy (serializer (*before));
	    if (after)
		    node->add_child_copy (serializer (*after));

            return *node;
        }

    protected:
        obj_T &obj;
        XMLNode *before, *after;
};

#endif // __lib_pbd_memento_h__
